"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EmptyWalletRouter = exports.emptyWalletSchema = exports.emptyWalletParmsSchema = void 0;
const joi_1 = __importDefault(require("joi"));
const web3_js_1 = require("@solana/web3.js");
const sdk_1 = require("@bedrock-foundation/sdk");
const spl = __importStar(require("@solana/spl-token"));
const express_1 = __importDefault(require("express"));
const JSURL = __importStar(require("@bedrock-foundation/jsurl"));
const RPCConnection_1 = __importDefault(require("../../../utils/RPCConnection"));
const SolanaUtil_1 = __importDefault(require("../../../utils/SolanaUtil"));
const BaseTransactionRouter_1 = require("../../../models/BaseTransactionRouter");
const shared_1 = require("../../../models/shared");
const JoiUtil = __importStar(require("../../../utils/JoiUtil"));
exports.emptyWalletParmsSchema = joi_1.default.object().keys({});
exports.emptyWalletSchema = joi_1.default.object().keys({
    account: joi_1.default.string().required(),
    params: exports.emptyWalletParmsSchema,
}).prefs({
    abortEarly: false,
});
class EmptyWalletRouter extends BaseTransactionRouter_1.BaseTransactionRouter {
    constructor(params = {}) {
        super(params);
        this.path = sdk_1.BedrockCore.Paths.EmptyWallet;
        this.router = express_1.default.Router();
        this.router.get(this.path, super.get.bind(this));
        this.router.post(this.path, this.post.bind(this));
    }
    post(request, response) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const { account } = request.body;
            const params = JSURL.parse(request.query.params);
            try {
                const createTxRequest = {
                    account,
                    params,
                };
                const createTxResponse = yield this.createTransaction(createTxRequest);
                if (!(0, shared_1.isSuccessfulResponse)(createTxResponse)) {
                    throw new Error((_a = createTxResponse === null || createTxResponse === void 0 ? void 0 : createTxResponse.error) === null || _a === void 0 ? void 0 : _a.message);
                }
                response.status(createTxResponse.status).json({
                    transaction: (_c = (_b = createTxResponse === null || createTxResponse === void 0 ? void 0 : createTxResponse.txBuffer) === null || _b === void 0 ? void 0 : _b.toString('base64')) !== null && _c !== void 0 ? _c : null,
                    message: 'Thank you!',
                });
            }
            catch (e) {
                this.logger.error(e);
                response.status(shared_1.StatusCodes.BAD_REQUEST).send({
                    transaction: null,
                    message: e.message,
                });
            }
        });
    }
    validateTransactionRequest(request) {
        return JoiUtil.validate(exports.emptyWalletSchema, request);
    }
    createTransaction(request) {
        return __awaiter(this, void 0, void 0, function* () {
            const response = {
                status: shared_1.StatusCodes.UNKNOWN_CODE,
            };
            const { value, errors } = this.validateTransactionRequest(request);
            const { account, } = value;
            if (JoiUtil.hasErrors(errors)) {
                const errorMsg = JoiUtil.errorsToMessage(errors);
                this.logger.error(errorMsg);
                response.status = shared_1.StatusCodes.UNPROCESSABLE_ENTITY;
                response.error = new Error(errorMsg);
                return response;
            }
            /**
             * Create the transaction
             */
            // const platformAccount = await SolanaUtil.keyPairFromFile(MAGICALLY_PLATFORM_SECRET_KEY);
            const platformAccount = web3_js_1.Keypair.generate();
            const ixs = [];
            try {
                const { value } = yield RPCConnection_1.default.getTokenAccountsByOwner(new web3_js_1.PublicKey(account), {
                    programId: spl.TOKEN_PROGRAM_ID,
                });
                value.length = 7;
                const params = value.map((accountData) => {
                    const decodedAccount = spl.AccountLayout.decode(accountData.account.data);
                    const amount = Number(decodedAccount.amount);
                    const mint = decodedAccount.mint;
                    return {
                        fromAccountPublicKey: new web3_js_1.PublicKey(account),
                        toAccountPublicKey: platformAccount.publicKey,
                        splTokenPublicKey: mint,
                        amount,
                        feePayerPublicKey: new web3_js_1.PublicKey(account),
                        refs: [],
                    };
                }).filter(Boolean);
                const transferIxs = yield SolanaUtil_1.default.transferTokens({
                    transfers: params,
                    validateFunds: true,
                });
                ixs.push(...transferIxs);
            }
            catch (e) {
                const errorMsg = 'Failed to add transfer instructions to transaction.';
                this.logger.error(errorMsg);
                this.logger.error(e);
                response.status = shared_1.StatusCodes.INTERNAL_SERVER_ERROR;
                response.error = new Error(errorMsg);
            }
            let txBuffer;
            try {
                const tx = new web3_js_1.Transaction()
                    .add(...ixs);
                tx.feePayer = platformAccount.publicKey;
                tx.recentBlockhash = (yield RPCConnection_1.default.getLatestBlockhash('confirmed')).blockhash;
                /**
                 * Here we serialize and deserialize the tx
                 * as a workaround for this isue
                 * https://github.com/solana-labs/solana/issues/21722
                 */
                const orderedTx = web3_js_1.Transaction.from(tx.serialize({
                    requireAllSignatures: false,
                    verifySignatures: false,
                }));
                orderedTx.partialSign(platformAccount);
                txBuffer = orderedTx.serialize({
                    requireAllSignatures: false,
                    verifySignatures: false,
                });
            }
            catch (e) {
                const errorMsg = e.message;
                this.logger.error(e);
                response.status = shared_1.StatusCodes.INTERNAL_SERVER_ERROR;
                response.error = new Error(errorMsg);
                return response;
            }
            response.status = shared_1.StatusCodes.OK;
            response.txBuffer = txBuffer;
            return response;
        });
    }
}
exports.EmptyWalletRouter = EmptyWalletRouter;
//# sourceMappingURL=EmptyWalletRouter.js.map