"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.GetReferenceStatusRouter = exports.getReferenceStatusParamsSchema = void 0;
const joi_1 = __importDefault(require("joi"));
const sdk_1 = require("@bedrock-foundation/sdk");
const express_1 = __importDefault(require("express"));
const web3_js_1 = require("@solana/web3.js");
const RPCConnection_1 = __importDefault(require("../../../utils/RPCConnection"));
const shared_1 = require("../../../models/shared");
const JoiUtil = __importStar(require("../../../utils/JoiUtil"));
exports.getReferenceStatusParamsSchema = joi_1.default.object().keys({
    ref: joi_1.default.string().required(),
}).prefs({
    abortEarly: false,
});
class GetReferenceStatusRouter {
    constructor(params) {
        var _a;
        this.logger = (_a = params.logger) !== null && _a !== void 0 ? _a : console;
        this.path = sdk_1.BedrockCore.Paths.ReferenceStatus;
        this.router = express_1.default.Router();
        this.router.get(this.path, this.get.bind(this));
    }
    get(request, response) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield this.status(request.query);
                if (!(0, shared_1.isSuccessfulResponse)(result)) {
                    throw new Error((_a = result === null || result === void 0 ? void 0 : result.error) === null || _a === void 0 ? void 0 : _a.message);
                }
                response.status(result.status).json(result.data);
            }
            catch (e) {
                this.logger.error(e);
                response.status(shared_1.StatusCodes.BAD_REQUEST).send({
                    status: sdk_1.TransactionStatuses.Unknown,
                    message: e.message,
                });
            }
        });
    }
    validateGetReferenceStatusParams(params) {
        return JoiUtil.validate(exports.getReferenceStatusParamsSchema, params);
    }
    status(params) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const response = {
                status: shared_1.StatusCodes.UNKNOWN_CODE,
            };
            const { value, errors } = this.validateGetReferenceStatusParams(params);
            const { ref, } = value;
            if (JoiUtil.hasErrors(errors)) {
                const errorMsg = JoiUtil.errorsToMessage(errors);
                this.logger.error(errorMsg);
                response.status = shared_1.StatusCodes.UNPROCESSABLE_ENTITY;
                response.error = new Error(errorMsg);
                return response;
            }
            try {
                const signatures = yield RPCConnection_1.default.getSignaturesForAddress(new web3_js_1.PublicKey(ref), {}, 'confirmed');
                const signature = (_b = (_a = signatures === null || signatures === void 0 ? void 0 : signatures[0]) === null || _a === void 0 ? void 0 : _a.signature) !== null && _b !== void 0 ? _b : null;
                response.data = {
                    signature,
                    status: signature
                        ? sdk_1.TransactionStatuses.Confirmed
                        : sdk_1.TransactionStatuses.Pending,
                };
                response.status = shared_1.StatusCodes.OK;
                return response;
            }
            catch (e) {
                const errorMsg = `Failed to get signature for ref ${ref}`;
                this.logger.error(errorMsg);
                this.logger.error(e);
                response.status = shared_1.StatusCodes.INTERNAL_SERVER_ERROR;
                response.error = new Error(errorMsg);
                return response;
            }
        });
    }
}
exports.GetReferenceStatusRouter = GetReferenceStatusRouter;
//# sourceMappingURL=GetReferenceStatusRouter.js.map