"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TransferRouter = exports.transferSchema = exports.transferParamsSchema = void 0;
const joi_1 = __importDefault(require("joi"));
const web3_js_1 = require("@solana/web3.js");
const sdk_1 = require("@bedrock-foundation/sdk");
const express_1 = __importDefault(require("express"));
const JSURL = __importStar(require("@bedrock-foundation/jsurl"));
const TokenGateUtil = __importStar(require("../../../utils/TokenGateUtil"));
const RPCConnection_1 = __importDefault(require("../../../utils/RPCConnection"));
const SolanaUtil_1 = __importDefault(require("../../../utils/SolanaUtil"));
const BaseTransactionRouter_1 = require("../../../models/BaseTransactionRouter");
const shared_1 = require("../../../models/shared");
const JoiUtil = __importStar(require("../../../utils/JoiUtil"));
const tokenGate = joi_1.default.object().keys({
    collection: joi_1.default.string().optional(),
    traits: joi_1.default.any(),
    discountPercentage: joi_1.default.number().optional(),
});
exports.transferParamsSchema = joi_1.default.object().keys({
    wallet: joi_1.default.string().required(),
    token: joi_1.default.string().required(),
    quantity: joi_1.default.number().optional(),
    gate: tokenGate,
    size: joi_1.default.number().optional(),
    icon: joi_1.default.string().optional(),
    label: joi_1.default.string().optional(),
    refs: joi_1.default.array().items(joi_1.default.string()).default([]),
});
exports.transferSchema = joi_1.default.object().keys({
    account: joi_1.default.string().required(),
    params: exports.transferParamsSchema,
}).prefs({
    abortEarly: false,
});
class TransferRouter extends BaseTransactionRouter_1.BaseTransactionRouter {
    constructor(params = {}) {
        super(params);
        this.path = sdk_1.BedrockCore.Paths.Transfer;
        this.router = express_1.default.Router();
        this.router.get(this.path, super.get.bind(this));
        this.router.post(this.path, this.post.bind(this));
    }
    post(req, res) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const { account } = req.body;
            const params = JSURL.parse(req.query.params);
            try {
                const request = {
                    account,
                    params,
                };
                const response = yield this.createTransaction(request);
                if (!(0, shared_1.isSuccessfulResponse)(response)) {
                    throw new Error((_a = response === null || response === void 0 ? void 0 : response.error) === null || _a === void 0 ? void 0 : _a.message);
                }
                res.status(response.status).json({
                    transaction: (_c = (_b = response === null || response === void 0 ? void 0 : response.txBuffer) === null || _b === void 0 ? void 0 : _b.toString('base64')) !== null && _c !== void 0 ? _c : null,
                    message: 'Thank you!',
                });
            }
            catch (e) {
                this.logger.error(e);
                res.status(shared_1.StatusCodes.BAD_REQUEST).send({
                    transaction: null,
                    message: e.message,
                });
            }
        });
    }
    validateTransactionRequest(request) {
        return JoiUtil.validate(exports.transferSchema, request);
    }
    createTransaction(request) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const response = {
                status: shared_1.StatusCodes.UNKNOWN_CODE,
            };
            const { value, errors } = this.validateTransactionRequest(request);
            const { account, params, } = value;
            if (JoiUtil.hasErrors(errors)) {
                const errorMsg = JoiUtil.errorsToMessage(errors);
                this.logger.error(errorMsg);
                response.status = shared_1.StatusCodes.UNPROCESSABLE_ENTITY;
                response.error = new Error(errorMsg);
                return response;
            }
            const { wallet, token, quantity, gate, size, refs, } = params;
            const customerPublicKey = new web3_js_1.PublicKey(account);
            const merchantPublicKey = new web3_js_1.PublicKey(wallet);
            /**
              * Apply the access gate if one is specified
              */
            try {
                const tokens = gate ? yield TokenGateUtil.applyAccessGate(gate, customerPublicKey) : [];
                if (gate && tokens.length === 0) {
                    const errorMsg = 'User does not have required NFT.';
                    this.logger.error(errorMsg);
                    response.status = shared_1.StatusCodes.UNAUTHORIZED;
                    response.error = new Error(errorMsg);
                    return response;
                }
            }
            catch (e) {
                const errorMsg = e.message;
                this.logger.error(e);
                response.status = shared_1.StatusCodes.INTERNAL_SERVER_ERROR;
                response.error = new Error(errorMsg);
                return response;
            }
            /**
             * Create the transaction
             */
            const ixs = [];
            try {
                if (token === sdk_1.TokenTypes.SOL) {
                    const amount = (_a = (size ? size * web3_js_1.LAMPORTS_PER_SOL : quantity)) !== null && _a !== void 0 ? _a : 0;
                    const merchantTransferIx = web3_js_1.SystemProgram.transfer({
                        fromPubkey: customerPublicKey,
                        toPubkey: merchantPublicKey,
                        lamports: amount,
                    });
                    refs === null || refs === void 0 ? void 0 : refs.forEach((ref) => {
                        merchantTransferIx.keys.push({ pubkey: new web3_js_1.PublicKey(ref), isWritable: false, isSigner: false });
                    });
                    ixs.push(merchantTransferIx);
                }
                else if (token === sdk_1.TokenTypes.USDC) {
                    const usdcTokenInfo = sdk_1.TokenInfo[sdk_1.TokenTypes.USDC];
                    const usdcTokenAddress = new web3_js_1.PublicKey(usdcTokenInfo.address);
                    const amount = (_b = (size ? size * (Math.pow(10, usdcTokenInfo.decimals)) : quantity)) !== null && _b !== void 0 ? _b : 0;
                    const params = [
                        {
                            fromAccountPublicKey: customerPublicKey,
                            toAccountPublicKey: merchantPublicKey,
                            splTokenPublicKey: usdcTokenAddress,
                            amount,
                            feePayerPublicKey: customerPublicKey,
                            refs: (_c = refs === null || refs === void 0 ? void 0 : refs.map((ref) => new web3_js_1.PublicKey(ref))) !== null && _c !== void 0 ? _c : [],
                        },
                    ];
                    const transferIxs = yield SolanaUtil_1.default.transferTokens({
                        transfers: params,
                        validateFunds: true,
                    });
                    ixs.push(...transferIxs);
                }
            }
            catch (e) {
                const errorMsg = 'Failed to add transfer instructions to transaction.';
                this.logger.error(errorMsg);
                this.logger.error(e);
                response.status = shared_1.StatusCodes.INTERNAL_SERVER_ERROR;
                response.error = new Error(errorMsg);
                return response;
            }
            let txBuffer;
            try {
                const tx = new web3_js_1.Transaction()
                    .add(...ixs);
                tx.feePayer = customerPublicKey;
                tx.recentBlockhash = (yield RPCConnection_1.default.getLatestBlockhash('confirmed')).blockhash;
                /**
                 * Here we serialize and deserialize the tx
                 * as a workaround for this isue
                 * https://github.com/solana-labs/solana/issues/21722
                 */
                const orderedTx = web3_js_1.Transaction.from(tx.serialize({
                    requireAllSignatures: false,
                    verifySignatures: false,
                }));
                txBuffer = orderedTx.serialize({
                    requireAllSignatures: false,
                    verifySignatures: false,
                });
            }
            catch (e) {
                const errorMsg = e.message;
                this.logger.error(e);
                response.status = shared_1.StatusCodes.INTERNAL_SERVER_ERROR;
                response.error = new Error(errorMsg);
                return response;
            }
            response.status = shared_1.StatusCodes.OK;
            response.txBuffer = txBuffer;
            return response;
        });
    }
}
exports.TransferRouter = TransferRouter;
//# sourceMappingURL=TransferRouter.js.map