"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.PollReferenceRouter = void 0;
const sdk_1 = require("@bedrock-foundation/sdk");
const express_1 = __importDefault(require("express"));
const web3_js_1 = require("@solana/web3.js");
const RPCConnection_1 = __importDefault(require("../../utils/RPCConnection"));
const BaseStatusRouter_1 = require("../../models/BaseStatusRouter");
const pollReference = new sdk_1.PollReferenceStatus();
class PollReferenceRouter extends BaseStatusRouter_1.BaseStatusRouter {
    constructor(params = {}) {
        super(params);
        this.path = pollReference.path;
        this.router = express_1.default.Router();
        this.router.get(this.path, this.get.bind(this));
    }
    get(request, response) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const result = yield this.status(request.query);
                if (!sdk_1.ErrorUtil.isSuccessfulResponse(result)) {
                    throw new Error((_a = result === null || result === void 0 ? void 0 : result.error) === null || _a === void 0 ? void 0 : _a.message);
                }
                response.status(result.status).json(result.data);
            }
            catch (e) {
                this.logger.error(e);
                response.status(sdk_1.StatusCodes.BAD_REQUEST).send({
                    message: e.message,
                });
            }
        });
    }
    status(params) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            const response = {
                status: sdk_1.StatusCodes.UNKNOWN_CODE,
            };
            const { value, errors } = pollReference.validateQueryStringParams(params);
            const { ref, } = value;
            if (sdk_1.JoiUtil.hasErrors(errors)) {
                const errorMsg = sdk_1.JoiUtil.errorsToMessage(errors);
                this.logger.error(errorMsg);
                response.status = sdk_1.StatusCodes.UNPROCESSABLE_ENTITY;
                response.error = new Error(errorMsg);
                return response;
            }
            try {
                const signatures = yield RPCConnection_1.default.getSignaturesForAddress(new web3_js_1.PublicKey(ref), {}, 'confirmed');
                response.data = { signature: (_b = (_a = signatures === null || signatures === void 0 ? void 0 : signatures[0]) === null || _a === void 0 ? void 0 : _a.signature) !== null && _b !== void 0 ? _b : null };
                response.status = sdk_1.StatusCodes.OK;
                return response;
            }
            catch (e) {
                const errorMsg = `Failed to get signature for ref ${ref}`;
                this.logger.error(errorMsg);
                this.logger.error(e);
                response.status = sdk_1.StatusCodes.INTERNAL_SERVER_ERROR;
                response.error = new Error(errorMsg);
                return response;
            }
        });
    }
}
exports.PollReferenceRouter = PollReferenceRouter;
//# sourceMappingURL=PollReferenceRouter.js.map