"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.AuthorizationRouter = exports.MEMO_PROGRAM_ID = exports.authorizationSchema = exports.authorizationParmsSchema = void 0;
const joi_1 = __importDefault(require("joi"));
const web3_js_1 = require("@solana/web3.js");
const sdk_1 = require("@bedrock-foundation/sdk");
const jsonwebtoken_1 = __importDefault(require("jsonwebtoken"));
const express_1 = __importDefault(require("express"));
const JSURL = __importStar(require("@bedrock-foundation/jsurl"));
const RPCConnection_1 = __importDefault(require("../../utils/RPCConnection"));
const BaseTransactionRouter_1 = require("../../models/BaseTransactionRouter");
const shared_1 = require("../../models/shared");
const JoiUtil = __importStar(require("../../utils/JoiUtil"));
exports.authorizationParmsSchema = joi_1.default.object().keys({
    refs: joi_1.default.array().items(joi_1.default.string()).default([]),
});
exports.authorizationSchema = joi_1.default.object().keys({
    account: joi_1.default.string().required(),
    params: exports.authorizationParmsSchema,
}).prefs({
    abortEarly: false,
});
exports.MEMO_PROGRAM_ID = new web3_js_1.PublicKey('MemoSq4gqABAXKb96qnH8TysNcWxMyWCqXgDLGmfcHr');
class AuthorizationRouter extends BaseTransactionRouter_1.BaseTransactionRouter {
    constructor(params) {
        super(params);
        this.path = sdk_1.BedrockCore.Paths.Authorization;
        this.noncePath = sdk_1.BedrockCore.Paths.AuthorizationNonce;
        this.router = express_1.default.Router();
        this.redis = params.redis;
        this.io = params.io;
        this.router.get(this.path, super.get.bind(this));
        this.router.get(this.noncePath, this.createNonce.bind(this));
        this.router.post(this.path, this.post.bind(this));
    }
    post(req, res) {
        var _a, _b, _c;
        return __awaiter(this, void 0, void 0, function* () {
            const { account } = req.body;
            const params = JSURL.parse(req.query.params);
            try {
                const request = {
                    account,
                    params,
                };
                const response = yield this.createTransaction(request);
                if (!(0, shared_1.isSuccessfulResponse)(response)) {
                    throw new Error((_a = response === null || response === void 0 ? void 0 : response.error) === null || _a === void 0 ? void 0 : _a.message);
                }
                res.status(response.status).json({
                    transaction: (_c = (_b = response === null || response === void 0 ? void 0 : response.txBuffer) === null || _b === void 0 ? void 0 : _b.toString('base64')) !== null && _c !== void 0 ? _c : null,
                    message: 'Authorize with Bedrock',
                });
            }
            catch (e) {
                this.logger.error(e);
                res.status(shared_1.StatusCodes.BAD_REQUEST).send({
                    transaction: null,
                    message: e.message,
                });
            }
        });
    }
    createNonce(req, res) {
        var _a, _b;
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const params = JSURL.parse(req.query.params);
                const ref = (_b = (_a = params.refs) === null || _a === void 0 ? void 0 : _a[0]) !== null && _b !== void 0 ? _b : '';
                const nonce = new web3_js_1.Keypair().publicKey.toBase58();
                this.redis.hSet('nonces', ref, nonce);
                res.status(shared_1.StatusCodes.OK).json({
                    nonce,
                });
            }
            catch (e) {
                this.logger.error(e);
                res.status(shared_1.StatusCodes.BAD_REQUEST).send({
                    nonce: null,
                    message: 'Failed to create nonce.',
                });
            }
        });
    }
    validateTransactionRequest(request) {
        return JoiUtil.validate(exports.authorizationSchema, request);
    }
    createTransaction(request) {
        var _a;
        return __awaiter(this, void 0, void 0, function* () {
            const response = {
                status: shared_1.StatusCodes.UNKNOWN_CODE,
            };
            const { value, errors } = this.validateTransactionRequest(request);
            const { account, params, } = value;
            if (JoiUtil.hasErrors(errors)) {
                const errorMsg = JoiUtil.errorsToMessage(errors);
                this.logger.error(errorMsg);
                response.status = shared_1.StatusCodes.UNPROCESSABLE_ENTITY;
                response.error = new Error(errorMsg);
                return response;
            }
            const { refs, } = params;
            const ref = (_a = refs === null || refs === void 0 ? void 0 : refs[0]) !== null && _a !== void 0 ? _a : '';
            const nonce = yield this.redis.hGet('nonces', ref);
            if (!nonce) {
                const errorMsg = 'Invalid request. Nonce not found.';
                this.logger.error(errorMsg);
                response.status = shared_1.StatusCodes.BAD_REQUEST;
                response.error = new Error(errorMsg);
                return response;
            }
            /**
             * Broadcast the scan to the client
             */
            const nonceSocketTopic = (0, sdk_1.createNonceSocketTopic)(nonce);
            const customerPublicKey = new web3_js_1.PublicKey(account);
            const wallet = customerPublicKey.toBase58();
            const authorizationData = {
                wallet,
                status: sdk_1.TransactionStatuses.Scanned,
                signature: null,
                token: null,
            };
            this.io.emit(nonceSocketTopic, authorizationData);
            /**
             * Create the transaction
             */
            let txBuffer;
            const keypair = web3_js_1.Keypair.generate();
            try {
                const memoIx = new web3_js_1.TransactionInstruction({
                    programId: exports.MEMO_PROGRAM_ID,
                    keys: [
                        {
                            pubkey: customerPublicKey,
                            isSigner: true,
                            isWritable: false,
                        },
                        {
                            pubkey: new web3_js_1.PublicKey(keypair.publicKey),
                            isSigner: true,
                            isWritable: false,
                        },
                    ],
                    data: Buffer.from('BEDROCK_AUTHORIZATION', 'utf8'),
                });
                const tx = new web3_js_1.Transaction().add(memoIx);
                tx.feePayer = customerPublicKey;
                tx.recentBlockhash = (yield RPCConnection_1.default.getLatestBlockhash('confirmed')).blockhash;
                /**
                 * Here we serialize and deserialize the tx
                 * as a workaround for this isue
                 * https://github.com/solana-labs/solana/issues/21722
                 */
                const orderedTx = web3_js_1.Transaction.from(tx.serialize({
                    requireAllSignatures: false,
                    verifySignatures: false,
                }));
                orderedTx.partialSign(keypair);
                txBuffer = orderedTx.serialize({
                    requireAllSignatures: false,
                    verifySignatures: false,
                });
                const createJwt = (signature, wallet) => {
                    return jsonwebtoken_1.default.sign({
                        signature,
                        wallet,
                    }, 'BEDROCK_AUTHORIZATION_SECRET');
                };
                const confirmTransaction = () => __awaiter(this, void 0, void 0, function* () {
                    try {
                        // try to confirm the transaction up to 30 times
                        for (let i = 0; i < 30; i++) {
                            try {
                                yield sdk_1.WaitUtil.wait(2000);
                                const signatures = yield RPCConnection_1.default.getSignaturesForAddress(keypair.publicKey, {}, 'confirmed');
                                if (signatures.length > 0) {
                                    const signature = signatures[0].signature;
                                    const authorizationData = {
                                        wallet,
                                        status: sdk_1.TransactionStatuses.Confirmed,
                                        signature,
                                        token: createJwt(signature, wallet),
                                    };
                                    this.io.emit(nonceSocketTopic, authorizationData);
                                    break;
                                }
                            }
                            catch (e) {
                                this.logger.log(e);
                                this.logger.log(`Failed to confirm tranaction with ref ${ref}`);
                            }
                        }
                    }
                    catch (e) {
                        this.logger.error('Failed to confirm transaction');
                        this.logger.error(e);
                    }
                });
                confirmTransaction();
            }
            catch (e) {
                const errorMsg = e.message;
                this.logger.error(e);
                response.status = shared_1.StatusCodes.INTERNAL_SERVER_ERROR;
                response.error = new Error(errorMsg);
                return response;
            }
            response.status = shared_1.StatusCodes.OK;
            response.txBuffer = txBuffer;
            return response;
        });
    }
}
exports.AuthorizationRouter = AuthorizationRouter;
//# sourceMappingURL=AuthorizationRouter.js.map