"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const web3_js_1 = require("@solana/web3.js");
const spl = __importStar(require("@solana/spl-token"));
const fs_extra_1 = __importDefault(require("fs-extra"));
const RPCConnection_1 = __importDefault(require("./RPCConnection"));
const keyPairFromFile = (filePath) => __awaiter(void 0, void 0, void 0, function* () {
    const file = yield fs_extra_1.default.readFile(filePath, 'utf8');
    const json = JSON.parse(file);
    return web3_js_1.Keypair.fromSecretKey(new Uint8Array(json.key));
});
const printTokenAccounts = (connection, publicKey, label) => __awaiter(void 0, void 0, void 0, function* () {
    const tokenAccounts = yield connection.getTokenAccountsByOwner(publicKey, {
        programId: spl.TOKEN_PROGRAM_ID,
    });
    console.log('\n\n');
    console.log(`For key ${label} ${publicKey}`);
    console.log('Token                                         Balance');
    console.log('------------------------------------------------------------');
    tokenAccounts.value.forEach((e) => {
        const accountInfo = spl.AccountLayout.decode(e.account.data);
        console.log(`${new web3_js_1.PublicKey(accountInfo.mint)}   ${accountInfo.amount}`);
    });
    console.log('\n\n');
});
const getRandomNFTMintFromAccount = (connection, account) => __awaiter(void 0, void 0, void 0, function* () {
    const { value } = yield connection.getTokenAccountsByOwner(account, {
        programId: spl.TOKEN_PROGRAM_ID,
    });
    const nfts = value.filter((accountData) => {
        const accountInfo = spl.AccountLayout.decode(accountData.account.data);
        return accountInfo.amount === BigInt(1);
    });
    const nft = nfts[Math.floor(Math.random() * nfts.length)];
    return spl.AccountLayout.decode(nft.account.data).mint;
});
const calculatePlatformFee = (amountInSol) => amountInSol * 0.01;
const calculateMerchantFee = (amountInSol) => amountInSol - calculatePlatformFee(amountInSol);
const getTokenAccountBalance = (connection, ataPublicKey) => __awaiter(void 0, void 0, void 0, function* () {
    const what = yield connection.getTokenAccountBalance(ataPublicKey);
    console.log(what);
    return BigInt(1);
});
const transferTokens = (params) => __awaiter(void 0, void 0, void 0, function* () {
    const { transfers } = params;
    /**
     * TODO: Talk with wallets about error handling
     *
     * The code below is for validating the balances
     * of each sending wallet before create the transactions
     * as the wallets will fail the transactions if the required
     * balance is not available.
     *
     * Unfortunately, there is no way to send an error message
     * to the wallet client letting them know that they do not have
     * the required funds.
     */
    // const transferAmount: Record<string, number> = transfers.reduce((cur, transfer) => {
    //   const key = `${transfer.fromAccountPublicKey.toString()}:${transfer.splTokenPublicKey.toString()}`;
    //   if(cur.hasOwnProperty(key)) {
    //     cur[key] += transfer.amount;
    //   } else {
    //     cur[key] = transfer.amount;
    //   }
    //   return cur;
    // }, {});
    // Object.keys(transferAmount).map(async (key) => {
    //   const parts = key.split(':');
    //   const fromAccountPublicKey = new PublicKey(parts[0]);
    //   const splTokenPublicKey = new PublicKey(parts[1]);
    //   const amount = transferAmount[key];
    //   const ata = await spl.getAssociatedTokenAddress(
    //     splTokenPublicKey,
    //     fromAccountPublicKey,
    //     false,
    //     spl.TOKEN_PROGRAM_ID,
    //     spl.ASSOCIATED_TOKEN_PROGRAM_ID,
    //   );
    //   const { value: { amount: balance } } = await RPCConnection.getTokenAccountBalance(ata)
    //   if (BigInt(balance) < BigInt(amount)) {
    //     throw new Error(`Not enough balance to transfer ${amount}`);
    //   }
    // });
    const createAtaAndTransferIxs = yield Promise.all(transfers.map(transferToken));
    return createAtaAndTransferIxs.flat();
});
const transferToken = (params) => __awaiter(void 0, void 0, void 0, function* () {
    const { fromAccountPublicKey, toAccountPublicKey, splTokenPublicKey, amount, feePayerPublicKey, refs, } = params;
    const transferIxs = [];
    const toAccountAtaPublicKey = yield spl.getAssociatedTokenAddress(splTokenPublicKey, toAccountPublicKey, false, spl.TOKEN_PROGRAM_ID, spl.ASSOCIATED_TOKEN_PROGRAM_ID);
    const account = yield RPCConnection_1.default.getAccountInfo(toAccountAtaPublicKey);
    if (account === null) {
        const createAtaIx = spl.createAssociatedTokenAccountInstruction(feePayerPublicKey, toAccountAtaPublicKey, toAccountPublicKey, splTokenPublicKey, spl.TOKEN_PROGRAM_ID, spl.ASSOCIATED_TOKEN_PROGRAM_ID);
        transferIxs.push(createAtaIx);
    }
    const fromAccountAtaPublicKey = yield spl.getAssociatedTokenAddress(splTokenPublicKey, fromAccountPublicKey, false, spl.TOKEN_PROGRAM_ID, spl.ASSOCIATED_TOKEN_PROGRAM_ID);
    const transferIx = spl.createTransferInstruction(fromAccountAtaPublicKey, toAccountAtaPublicKey, fromAccountPublicKey, amount);
    refs.forEach((ref) => {
        transferIx.keys.push({ pubkey: ref, isWritable: false, isSigner: false });
    });
    transferIxs.push(transferIx);
    return transferIxs;
});
const SolanaUtil = {
    keyPairFromFile,
    printTokenAccounts,
    getRandomNFTMintFromAccount,
    calculatePlatformFee,
    calculateMerchantFee,
    getTokenAccountBalance,
    transferTokens,
    transferToken,
};
exports.default = SolanaUtil;
//# sourceMappingURL=SolanaUtil.js.map