"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.applyAccessGate = void 0;
const js_next_1 = require("@metaplex-foundation/js-next");
const RPCConnection_1 = __importDefault(require("./RPCConnection"));
const metaplex = js_next_1.Metaplex.make(RPCConnection_1.default);
function applyAccessGate(gate, publicKey) {
    return __awaiter(this, void 0, void 0, function* () {
        const { collectionId, firstCreatorId, updateAuthorityId, traits, } = gate;
        const nfts = yield metaplex.nfts().findAllByOwner(publicKey);
        const selectedNfts = nfts.filter((nft) => {
            var _a, _b, _c, _d, _e, _f, _g, _h;
            if (collectionId) {
                return (_c = collectionId === ((_b = (_a = nft === null || nft === void 0 ? void 0 : nft.collection) === null || _a === void 0 ? void 0 : _a.key) === null || _b === void 0 ? void 0 : _b.toBase58())) !== null && _c !== void 0 ? _c : null;
            }
            if (firstCreatorId) {
                return (_f = firstCreatorId === ((_e = (_d = nft === null || nft === void 0 ? void 0 : nft.creators) === null || _d === void 0 ? void 0 : _d[0].address) === null || _e === void 0 ? void 0 : _e.toBase58())) !== null && _f !== void 0 ? _f : null;
            }
            if (updateAuthorityId) {
                return (_h = updateAuthorityId === ((_g = nft === null || nft === void 0 ? void 0 : nft.updateAuthority) === null || _g === void 0 ? void 0 : _g.toBase58())) !== null && _h !== void 0 ? _h : null;
            }
            return false;
        });
        const convertTraits = (traits) => {
            return Object.entries(traits).map(([key, value]) => {
                return `${key}:${value}`;
            }).sort();
        };
        const convertMetaplexAttributesToTraits = (attributes) => {
            return attributes.reduce((cur, next) => {
                const key = next.trait_type;
                if (!key)
                    return cur;
                cur[key] = next.value;
                return cur;
            }, {});
        };
        const tokens = (yield Promise.all(selectedNfts.map((nft) => __awaiter(this, void 0, void 0, function* () {
            var _a, _b;
            const metadata = yield nft.metadataTask.run();
            return {
                name: nft.name,
                mint: (_a = nft === null || nft === void 0 ? void 0 : nft.mint) === null || _a === void 0 ? void 0 : _a.toBase58(),
                image: metadata.image,
                traits: convertMetaplexAttributesToTraits((_b = metadata.attributes) !== null && _b !== void 0 ? _b : []),
            };
        })))).filter((nft) => {
            // if no traits are specified, return all tokens
            if (!traits || Object.keys(traits !== null && traits !== void 0 ? traits : {}).length === 0) {
                return nft;
            }
            // if traits are specified, return tokens that match the traits
            const specifiedTraits = convertTraits(traits);
            const tokenTraits = convertTraits(nft.traits);
            const invalidTrait = specifiedTraits.find((trait) => {
                return !tokenTraits.includes(trait);
            });
            return !invalidTrait;
        });
        return tokens;
    });
}
exports.applyAccessGate = applyAccessGate;
//# sourceMappingURL=TokenGateUtil.js.map