import Joi from 'joi';
import { BedrockCore, TransactionStatuses, } from '@bedrock-foundation/sdk';
import express from 'express';
import { PublicKey } from '@solana/web3.js';
import RPCConnection from './../../../utils/RPCConnection.mjs';
import { isSuccessfulResponse, StatusCodes, } from './../../../models/shared.mjs';
import * as JoiUtil from './../../../utils/JoiUtil.mjs';
export const getReferenceStatusParamsSchema = Joi.object().keys({
    ref: Joi.string().required(),
}).prefs({
    abortEarly: false,
});
export class GetReferenceStatusRouter {
    constructor(params) {
        var _a;
        this.logger = (_a = params.logger) !== null && _a !== void 0 ? _a : console;
        this.path = BedrockCore.Paths.ReferenceStatus;
        this.router = express.Router();
        this.router.get(this.path, this.get.bind(this));
    }
    async get(request, response) {
        var _a;
        try {
            const result = await this.status(request.query);
            if (!isSuccessfulResponse(result)) {
                throw new Error((_a = result === null || result === void 0 ? void 0 : result.error) === null || _a === void 0 ? void 0 : _a.message);
            }
            response.status(result.status).json(result.data);
        }
        catch (e) {
            this.logger.error(e);
            response.status(StatusCodes.BAD_REQUEST).send({
                status: TransactionStatuses.Unknown,
                message: e.message,
            });
        }
    }
    validateGetReferenceStatusParams(params) {
        return JoiUtil.validate(getReferenceStatusParamsSchema, params);
    }
    async status(params) {
        var _a, _b;
        const response = {
            status: StatusCodes.UNKNOWN_CODE,
        };
        const { value, errors } = this.validateGetReferenceStatusParams(params);
        const { ref, } = value;
        if (JoiUtil.hasErrors(errors)) {
            const errorMsg = JoiUtil.errorsToMessage(errors);
            this.logger.error(errorMsg);
            response.status = StatusCodes.UNPROCESSABLE_ENTITY;
            response.error = new Error(errorMsg);
            return response;
        }
        try {
            const signatures = await RPCConnection.getSignaturesForAddress(new PublicKey(ref), {}, 'confirmed');
            const signature = (_b = (_a = signatures === null || signatures === void 0 ? void 0 : signatures[0]) === null || _a === void 0 ? void 0 : _a.signature) !== null && _b !== void 0 ? _b : null;
            response.data = {
                signature,
                status: signature
                    ? TransactionStatuses.Confirmed
                    : TransactionStatuses.Pending,
            };
            response.status = StatusCodes.OK;
            return response;
        }
        catch (e) {
            const errorMsg = `Failed to get signature for ref ${ref}`;
            this.logger.error(errorMsg);
            this.logger.error(e);
            response.status = StatusCodes.INTERNAL_SERVER_ERROR;
            response.error = new Error(errorMsg);
            return response;
        }
    }
}
//# sourceMappingURL=GetReferenceStatusRouter.js.map