import { PollReferenceStatus, StatusCodes, JoiUtil, ErrorUtil, } from '@bedrock-foundation/sdk';
import express from 'express';
import { PublicKey } from '@solana/web3.js';
import RPCConnection from './../../utils/RPCConnection.mjs';
import { BaseStatusRouter, } from './../../models/BaseStatusRouter.mjs';
const pollReference = new PollReferenceStatus();
export class PollReferenceRouter extends BaseStatusRouter {
    constructor(params = {}) {
        super(params);
        this.path = pollReference.path;
        this.router = express.Router();
        this.router.get(this.path, this.get.bind(this));
    }
    async get(request, response) {
        var _a;
        try {
            const result = await this.status(request.query);
            if (!ErrorUtil.isSuccessfulResponse(result)) {
                throw new Error((_a = result === null || result === void 0 ? void 0 : result.error) === null || _a === void 0 ? void 0 : _a.message);
            }
            response.status(result.status).json(result.data);
        }
        catch (e) {
            this.logger.error(e);
            response.status(StatusCodes.BAD_REQUEST).send({
                message: e.message,
            });
        }
    }
    async status(params) {
        var _a, _b;
        const response = {
            status: StatusCodes.UNKNOWN_CODE,
        };
        const { value, errors } = pollReference.validateQueryStringParams(params);
        const { ref, } = value;
        if (JoiUtil.hasErrors(errors)) {
            const errorMsg = JoiUtil.errorsToMessage(errors);
            this.logger.error(errorMsg);
            response.status = StatusCodes.UNPROCESSABLE_ENTITY;
            response.error = new Error(errorMsg);
            return response;
        }
        try {
            const signatures = await RPCConnection.getSignaturesForAddress(new PublicKey(ref), {}, 'confirmed');
            response.data = { signature: (_b = (_a = signatures === null || signatures === void 0 ? void 0 : signatures[0]) === null || _a === void 0 ? void 0 : _a.signature) !== null && _b !== void 0 ? _b : null };
            response.status = StatusCodes.OK;
            return response;
        }
        catch (e) {
            const errorMsg = `Failed to get signature for ref ${ref}`;
            this.logger.error(errorMsg);
            this.logger.error(e);
            response.status = StatusCodes.INTERNAL_SERVER_ERROR;
            response.error = new Error(errorMsg);
            return response;
        }
    }
}
//# sourceMappingURL=PollReferenceRouter.js.map