import { Metaplex } from '@metaplex-foundation/js-next';
import RPCConnection from './RPCConnection.mjs';
const metaplex = Metaplex.make(RPCConnection);
export async function applyAccessGate(gate, publicKey) {
    const { collectionId, firstCreatorId, updateAuthorityId, traits, } = gate;
    const nfts = await metaplex.nfts().findAllByOwner(publicKey);
    const selectedNfts = nfts.filter((nft) => {
        var _a, _b, _c, _d, _e, _f, _g, _h;
        if (collectionId) {
            return (_c = collectionId === ((_b = (_a = nft === null || nft === void 0 ? void 0 : nft.collection) === null || _a === void 0 ? void 0 : _a.key) === null || _b === void 0 ? void 0 : _b.toBase58())) !== null && _c !== void 0 ? _c : null;
        }
        if (firstCreatorId) {
            return (_f = firstCreatorId === ((_e = (_d = nft === null || nft === void 0 ? void 0 : nft.creators) === null || _d === void 0 ? void 0 : _d[0].address) === null || _e === void 0 ? void 0 : _e.toBase58())) !== null && _f !== void 0 ? _f : null;
        }
        if (updateAuthorityId) {
            return (_h = updateAuthorityId === ((_g = nft === null || nft === void 0 ? void 0 : nft.updateAuthority) === null || _g === void 0 ? void 0 : _g.toBase58())) !== null && _h !== void 0 ? _h : null;
        }
        return false;
    });
    const convertTraits = (traits) => {
        return Object.entries(traits).map(([key, value]) => {
            return `${key}:${value}`;
        }).sort();
    };
    const convertMetaplexAttributesToTraits = (attributes) => {
        return attributes.reduce((cur, next) => {
            const key = next.trait_type;
            if (!key)
                return cur;
            cur[key] = next.value;
            return cur;
        }, {});
    };
    const tokens = (await Promise.all(selectedNfts.map(async (nft) => {
        var _a, _b;
        const metadata = await nft.metadataTask.run();
        return {
            name: nft.name,
            mint: (_a = nft === null || nft === void 0 ? void 0 : nft.mint) === null || _a === void 0 ? void 0 : _a.toBase58(),
            image: metadata.image,
            traits: convertMetaplexAttributesToTraits((_b = metadata.attributes) !== null && _b !== void 0 ? _b : []),
        };
    }))).filter((nft) => {
        // if no traits are specified, return all tokens
        if (!traits || Object.keys(traits !== null && traits !== void 0 ? traits : {}).length === 0) {
            return nft;
        }
        // if traits are specified, return tokens that match the traits
        const specifiedTraits = convertTraits(traits);
        const tokenTraits = convertTraits(nft.traits);
        const invalidTrait = specifiedTraits.find((trait) => {
            return !tokenTraits.includes(trait);
        });
        return !invalidTrait;
    });
    return tokens;
}
//# sourceMappingURL=TokenGateUtil.js.map