/// <reference types="node" />
import { JsUrl } from '@bedrock-foundation/jsurl';
import { Request, Response } from 'express';
export declare type HTTPRequest<PostBody, QueryStringParams> = Request<{}, {}, PostBody, QueryStringParams>;
export declare type HTTPResponse<ResponseData> = Response<ResponseData>;
export declare type TransactionRequest<TransactionRequestParams> = HTTPRequest<{
    account: string;
}, {
    params: JsUrl<TransactionRequestParams>;
}>;
export declare type TransactionResponse = HTTPResponse<{
    transaction: string | null;
    message?: string;
}>;
export declare type MetadataRequest<MetadataRequestParams> = HTTPRequest<never, {
    params: JsUrl<MetadataRequestParams>;
}>;
export declare type MetadataResponse = HTTPResponse<{
    icon: string;
    label: string;
}>;
export declare type NonceRequest<NonceRequestParams> = HTTPRequest<never, {
    params: JsUrl<NonceRequestParams>;
}>;
export declare type NonceResponse = HTTPResponse<{
    nonce: string | null;
    message?: string;
}>;
export declare type StatusRequest<StatusRequestParams> = HTTPRequest<never, StatusRequestParams>;
export declare type StatusResponse = HTTPResponse<{
    nonce: string | null;
    message?: string;
}>;
export interface CreateTransactionRequest<Params> {
    account: string;
    params: Params;
}
export interface CreateTransactionResponse {
    txBuffer?: Buffer;
    error?: Error;
    status: StatusCodes;
}
export declare enum StatusCodes {
    UNKNOWN_CODE = 0,
    OK = 200,
    BAD_REQUEST = 400,
    UNAUTHORIZED = 401,
    FORBIDDEN = 403,
    NOT_FOUND = 404,
    UNPROCESSABLE_ENTITY = 422,
    INTERNAL_SERVER_ERROR = 500,
    GATEWAY_TIMEOUT = 504
}
export declare const isSuccessfulResponse: (response: CreateTransactionResponse) => boolean;
